@@ -10,6 +10,7 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <errno.h>
 #include <ctype.h>
 
@@ -23,6 +24,16 @@
 #include <sys/wait.h>
 #include <sys/time.h>
 
+#ifdef _AIX
+#  include <sys/select.h>
+#endif
+
+#ifdef _AIX
+#  include <sys/select.h>
+#endif
+#ifdef VMS
+#  include <sys/socket.h>     /* for fd_set */
+#endif
 #define IN_LIBIDENT_SRC
 #include "ident.h"
 
@@ -62,12 +73,12 @@ int id_parse __P7(ident_t *, id,
     {
 	FD_ZERO(&rs);
 	FD_SET(id->fd, &rs);
-	
-	if ((res = select(FD_SETSIZE,
-			  &rs,
-			  (fd_set *)0,
-			  (fd_set *)0,
-			  timeout)) < 0)
+
+#ifdef __hpux
+	if ((res = select(FD_SETSIZE, (int *) &rs, (int *)0, (int *)0, timeout)) < 0)
+#else
+	if ((res = select(FD_SETSIZE, &rs, (fd_set *)0, (fd_set *)0, timeout)) < 0)
+#endif
 	    return -1;
 	
 	if (res == 0)
@@ -77,9 +88,10 @@ int id_parse __P7(ident_t *, id,
 	}
     }
     
+    /* Every octal value is allowed except 0, \n and \r */
     while (pos < sizeof(id->buf) &&
 	   (res = read(id->fd, id->buf + pos, 1)) == 1 &&
-	   id->buf[pos] != '\n')
+	   id->buf[pos] != '\n' && id->buf[pos] != '\r')
 	pos++;
     
     if (res < 0)
@@ -91,22 +103,26 @@ int id_parse __P7(ident_t *, id,
 	return -1;
     }
     
-    if (id->buf[pos] != '\n')
-	return 0;
+    if (id->buf[pos] != '\n' && id->buf[pos] != '\r')
+	return 0;		/* Not properly terminated string */
     
     id->buf[pos++] = '\0';
     
     /*
     ** Get first field (<lport> , <fport>)
     */
-    cp = _id_strtok(id->buf, ":", &c);
+    cp = id_strtok(id->buf, ":", &c);
     if (!cp)
 	return -2;
     
     if (sscanf(cp, " %d , %d", &lp, &fp) != 2)
     {
 	if (identifier)
-	    *identifier = _id_strdup(cp);
+	{
+	    *identifier = id_strdup(cp);
+	    if (*identifier == NULL)
+	        return -4;
+	}
 	return -2;
     }
     
@@ -118,18 +134,22 @@ int id_parse __P7(ident_t *, id,
     /*
     ** Get second field (USERID or ERROR)
     */
-    cp = _id_strtok((char *)0, ":", &c);
+    cp = id_strtok((char *)0, ":", &c);
     if (!cp)
 	return -2;
     
     if (strcmp(cp, "ERROR") == 0)
     {
-	cp = _id_strtok((char *)0, "\n\r", &c);
+	cp = id_strtok((char *)0, "\n\r", &c);
 	if (!cp)
 	    return -2;
 	
 	if (identifier)
-	    *identifier = _id_strdup(cp);
+	{
+	    *identifier = id_strdup(cp);
+	    if (*identifier == NULL)
+	        return -4;
+	}
 	
 	return 2;
     }
@@ -138,46 +158,62 @@ int id_parse __P7(ident_t *, id,
 	/*
 	** Get first subfield of third field <opsys>
 	*/
-	cp = _id_strtok((char *) 0, ",:", &c);
+	cp = id_strtok((char *) 0, ",:", &c);
 	if (!cp)
 	    return -2;
 	
 	if (opsys)
-	    *opsys = _id_strdup(cp);
+	{
+	    *opsys = id_strdup(cp);
+	    if (*opsys == NULL)
+	        return -4;
+	}
 	
 	/*
 	** We have a second subfield (<charset>)
 	*/
 	if (c == ',')
 	{
-	    cp = _id_strtok((char *)0, ":", &c);
+	    cp = id_strtok((char *)0, ":", &c);
 	    if (!cp)
 		return -2;
 	    
 	    tmp_charset = cp;
 	    if (charset)
-		*charset = _id_strdup(cp);
+	    {
+		*charset = id_strdup(cp);
+		if (*charset == NULL)
+		    return -4;
+	    }
 	    
 	    /*
 	    ** We have even more subfields - ignore them
 	    */
 	    if (c == ',')
-		_id_strtok((char *)0, ":", &c);
+		id_strtok((char *)0, ":", &c);
 	}
 	
 	if (tmp_charset && strcmp(tmp_charset, "OCTET") == 0)
-	    cp = _id_strtok((char *)0, (char *)0, &c);
+	    cp = id_strtok((char *)0, (char *)0, &c);
 	else
-	    cp = _id_strtok((char *)0, "\n\r", &c);
+	    cp = id_strtok((char *)0, "\n\r", &c);
 	
 	if (identifier)
-	    *identifier = _id_strdup(cp);
+	{
+	    *identifier = id_strdup(cp);
+	    if (*identifier == NULL)
+	        return -4;
+	}
 	return 1;
     }
     else
     {
 	if (identifier)
-	    *identifier = _id_strdup(cp);
+	{
+	    *identifier = id_strdup(cp);
+	    if (*identifier == NULL)
+	        return -4;
+	}
 	return -3;
     }
 }
